@@ -33,6 +33,7 @@ import (
 	"github.com/k1LoW/tbls/output/gviz"
 	"github.com/k1LoW/tbls/output/json"
 	"github.com/k1LoW/tbls/output/md"
+	"github.com/k1LoW/tbls/output/mermaid"
 	"github.com/k1LoW/tbls/output/plantuml"
 	"github.com/k1LoW/tbls/output/xlsx"
 	"github.com/k1LoW/tbls/output/yaml"
@@ -43,7 +44,6 @@ import (
 var (
 	format   string
 	outPath  string
-	tables   []string
 	distance int
 )
 
@@ -98,6 +98,8 @@ var outCmd = &cobra.Command{
 			o = xlsx.New(c)
 		case "plantuml":
 			o = plantuml.New(c)
+		case "mermaid":
+			o = mermaid.New(c)
 		case "png", "svg", "jpg":
 			c.ER.Format = format
 			o = gviz.New(c)
@@ -145,6 +147,7 @@ func loadOutArgs(args []string) ([]config.Option, error) {
 
 	options = append(options, config.Include(append(tables, includes...)))
 	options = append(options, config.Exclude(excludes))
+	options = append(options, config.IncludeLabels(labels))
 
 	if len(args) == 1 {
 		options = append(options, config.DSNURL(args[0]))
@@ -158,9 +161,10 @@ func init() {
 	outCmd.Flags().StringVarP(&configPath, "config", "c", "", "config file path")
 	outCmd.Flags().StringVarP(&format, "format", "t", "json", "output format")
 	outCmd.Flags().StringVarP(&outPath, "out", "o", "", "output file path")
-	outCmd.Flags().StringSliceVarP(&tables, "table", "", []string{}, "target table")
+	outCmd.Flags().StringSliceVarP(&tables, "table", "", []string{}, "target table (tables to include)")
 	outCmd.Flags().StringSliceVarP(&includes, "include", "", []string{}, "tables to include")
 	outCmd.Flags().StringSliceVarP(&excludes, "exclude", "", []string{}, "tables to exclude")
+	outCmd.Flags().StringSliceVarP(&labels, "label", "", []string{}, "table labels to be included")
 	outCmd.Flags().IntVarP(&distance, "distance", "", 0, "distance between related tables to be displayed")
 	outCmd.Flags().StringVarP(&when, "when", "", "", "command execute condition")
 }
