@@ -24,6 +24,8 @@ var DefaultConfigFilePaths = []string{".tbls.yml", "tbls.yml"}
 // DefaultERFormat is the default ER diagram format
 const DefaultERFormat = "svg"
 
+var SupportERFormat = []string{"png", "jpg", "svg", "mermaid"}
+
 const SchemaFileName = "schema.json"
 
 // DefaultERDistance is the default distance between tables that display relations in the ER
@@ -53,6 +55,10 @@ type Config struct {
 	RequiredVersion        string                 `yaml:"requiredVersion,omitempty"`
 	DisableOutputSchema    bool                   `yaml:"disableOutputSchema,omitempty"`
 	MergedDict             dict.Dict              `yaml:"-"`
+
+	// Table labels to be included
+	includeLabels []string
+
 	// Path of config file
 	Path string `yaml:"-"`
 	root string `yaml:"-"`
@@ -77,17 +83,21 @@ type ER struct {
 	Skip     bool   `yaml:"skip,omitempty"`
 	Format   string `yaml:"format,omitempty"`
 	Comment  bool   `yaml:"comment,omitempty"`
+	HideDef  bool   `yaml:"hideDef,omitempty"`
 	Distance *int   `yaml:"distance,omitempty"`
 	Font     string `yaml:"font,omitempty"`
 }
 
 // AdditionalRelation is the struct for table relation from yaml
 type AdditionalRelation struct {
-	Table         string   `yaml:"table"`
-	Columns       []string `yaml:"columns"`
-	ParentTable   string   `yaml:"parentTable"`
-	ParentColumns []string `yaml:"parentColumns"`
-	Def           string   `yaml:"def,omitempty"`
+	Table             string   `yaml:"table"`
+	Columns           []string `yaml:"columns"`
+	Cardinality       string   `yaml:"cardinality,omitempty"`
+	ParentTable       string   `yaml:"parentTable"`
+	ParentColumns     []string `yaml:"parentColumns"`
+	ParentCardinality string   `yaml:"parentCardinality,omitempty"`
+	Def               string   `yaml:"def,omitempty"`
+	Override          bool     `yaml:"override,omitempty"`
 }
 
 // AdditionalComment is the struct for table relation from yaml
@@ -202,6 +212,16 @@ func Exclude(e []string) Option {
 	}
 }
 
+// IncludeLabels return Option set Config.includeLabels
+func IncludeLabels(l []string) Option {
+	return func(c *Config) error {
+		if len(l) > 0 {
+			c.includeLabels = l
+		}
+		return nil
+	}
+}
+
 // New return Config
 func New() (*Config, error) {
 	c := Config{}
@@ -212,7 +232,7 @@ func New() (*Config, error) {
 	return &c, nil
 }
 
-// Load load config with all method
+// Load config with all method
 func (c *Config) Load(configPath string, options ...Option) error {
 	if err := c.LoadConfigFile(configPath); err != nil {
 		return err
@@ -230,7 +250,7 @@ func (c *Config) Load(configPath string, options ...Option) error {
 		return err
 	}
 
-	if err := c.checkVersion(ver.Version); err != nil {
+	if err := c.validate(); err != nil {
 		return err
 	}
 
@@ -286,6 +306,16 @@ func (c *Config) checkVersion(sv string) error {
 	return nil
 }
 
+func (c *Config) validate() error {
+	if err := c.checkVersion(ver.Version); err != nil {
+		return err
+	}
+	if !contains(SupportERFormat, c.ER.Format) {
+		return fmt.Errorf("unsupported ER format: %s", c.ER.Format)
+	}
+	return nil
+}
+
 // LoadEnviron load environment variables
 func (c *Config) LoadEnviron() error {
 	dsn := os.Getenv("TBLS_DSN")
@@ -352,6 +382,12 @@ func (c *Config) ModifySchema(s *schema.Schema) error {
 			s.Labels = s.Labels.Merge(l)
 		}
 	}
+	if err := detectCardinality(s); err != nil {
+		return err
+	}
+	if err := detectPKFK(s); err != nil {
+		return err
+	}
 	err := c.MergeAdditionalData(s)
 	if err != nil {
 		return err
@@ -374,6 +410,9 @@ func (c *Config) ModifySchema(s *schema.Schema) error {
 		mergeDetectedRelations(s, strategy)
 	}
 	c.mergeDictFromSchema(s)
+	if err := detectCardinality(s); err != nil {
+		return err
+	}
 	return nil
 }
 
@@ -400,21 +439,27 @@ func (c *Config) FilterTables(s *schema.Schema) error {
 	for _, t := range s.Tables {
 		li, mi := matchLength(i, t.Name)
 		le, me := matchLength(e, t.Name)
+		ml := matchLabels(c.includeLabels, t.Labels)
 		switch {
-		case len(c.Include) == 0:
-			if me {
-				excludes = append(excludes, t)
-				continue
-			}
-			includes = append(includes, t)
 		case mi:
 			if me && li < le {
 				excludes = append(excludes, t)
 				continue
 			}
 			includes = append(includes, t)
+		case ml:
+			if me {
+				excludes = append(excludes, t)
+				continue
+			}
+			includes = append(includes, t)
+		case len(c.Include) == 0 && len(c.includeLabels) == 0:
+			if me {
+				excludes = append(excludes, t)
+				continue
+			}
+			includes = append(includes, t)
 		default:
-			// c.Include > 0 && !mi
 			excludes = append(excludes, t)
 		}
 	}
@@ -511,6 +556,16 @@ func (c *Config) SchemaFilePath() string {
 	return filepath.Join(c.DocPath, SchemaFileName)
 }
 
+func (c *Config) NeedToGenerateERImages() bool {
+	if c.ER.Skip {
+		return false
+	}
+	if c.ER.Format == "mermaid" {
+		return false
+	}
+	return true
+}
+
 func mergeAdditionalRelations(s *schema.Schema, relations []AdditionalRelation) error {
 	for _, r := range relations {
 		relation := &schema.Relation{
@@ -547,7 +602,25 @@ func mergeAdditionalRelations(s *schema.Schema, relations []AdditionalRelation)
 			column.ChildRelations = append(column.ChildRelations, relation)
 		}
 
-		s.Relations = append(s.Relations, relation)
+		if r.Override {
+			cr, err := s.FindRelation(relation.Columns, relation.ParentColumns)
+			if err != nil {
+				s.Relations = append(s.Relations, relation)
+			} else {
+				cr.Virtual = true
+				cr.Def = r.Def
+				cr.Cardinality, err = schema.ToCardinality(r.Cardinality)
+				if err != nil {
+					return errors.Wrap(err, "failed to add relation")
+				}
+				cr.ParentCardinality, err = schema.ToCardinality(r.ParentCardinality)
+				if err != nil {
+					return errors.Wrap(err, "failed to add relation")
+				}
+			}
+		} else {
+			s.Relations = append(s.Relations, relation)
+		}
 	}
 	return nil
 }
@@ -652,7 +725,103 @@ func matchLength(s []string, e string) (int, bool) {
 	return 0, false
 }
 
+func detectCardinality(s *schema.Schema) error {
+	// This function should be applied to the completed schema
+	for _, r := range s.Relations {
+		// child
+		if r.Cardinality == schema.UnknownCardinality {
+			unique := false
+			columns := []string{}
+			for _, c := range r.Columns {
+				columns = append(columns, c.Name)
+			}
+		LL:
+			for _, c := range r.Table.Constraints {
+				if len(columns) != len(c.Columns) {
+					continue
+				}
+				for _, cc := range c.Columns {
+					if !contains(columns, cc) {
+						continue LL
+					}
+				}
+				if strings.Contains(strings.ToUpper(c.Def), "UNIQUE") || strings.Contains(strings.ToUpper(c.Def), "PRIMARY KEY") {
+					unique = true
+				}
+			}
+			if unique {
+				r.Cardinality = schema.ZeroOrOne
+			} else {
+				r.Cardinality = schema.ZeroOrMore
+			}
+		}
+
+		// parent
+		if r.ParentCardinality == schema.UnknownCardinality {
+			// whether the child colums are nullable or not.
+			nullable := true
+			for _, c := range r.Columns {
+				if !c.Nullable {
+					nullable = false
+				}
+			}
+
+			if nullable {
+				r.ParentCardinality = schema.ZeroOrOne
+			} else {
+				r.ParentCardinality = schema.ExactlyOne
+			}
+		}
+	}
+	return nil
+}
+
+func detectPKFK(s *schema.Schema) error {
+	for _, t := range s.Tables {
+		// PRIMARY KEY
+		for _, i := range t.Indexes {
+			if !strings.Contains(i.Def, "PRIMARY") {
+				continue
+			}
+			for _, c := range i.Columns {
+				column, err := t.FindColumnByName(c)
+				if err != nil {
+					return err
+				}
+				column.PK = true
+			}
+		}
+		// Foreign Key (Relations)
+		for _, c := range t.Columns {
+			if len(c.ParentRelations) > 0 && !c.PK {
+				c.FK = true
+			}
+		}
+	}
+	return nil
+}
+
+func matchLabels(il []string, l schema.Labels) bool {
+	for _, ll := range l {
+		for _, ill := range il {
+			if ll.Name == ill {
+				return true
+			}
+		}
+	}
+	return false
+}
+
 func match(s []string, e string) bool {
 	_, m := matchLength(s, e)
 	return m
 }
+
+func contains(s []string, e string) bool {
+	for _, v := range s {
+		if e == v {
+			return true
+		}
+	}
+	return false
+}
