@@ -84,6 +84,8 @@ type Column struct {
 	Labels          Labels          `json:"labels,omitempty"`
 	ParentRelations []*Relation     `json:"-"`
 	ChildRelations  []*Relation     `json:"-"`
+	PK              bool            `json:"-"`
+	FK              bool            `json:"-"`
 }
 
 // Table is the struct for database table
@@ -103,12 +105,14 @@ type Table struct {
 
 // Relation is the struct for table relation
 type Relation struct {
-	Table         *Table    `json:"table"`
-	Columns       []*Column `json:"columns"`
-	ParentTable   *Table    `json:"parent_table" yaml:"parentTable"`
-	ParentColumns []*Column `json:"parent_columns" yaml:"parentColumns"`
-	Def           string    `json:"def"`
-	Virtual       bool      `json:"virtual"`
+	Table             *Table      `json:"table"`
+	Columns           []*Column   `json:"columns"`
+	ParentTable       *Table      `json:"parent_table" yaml:"parentTable"`
+	ParentColumns     []*Column   `json:"parent_columns" yaml:"parentColumns"`
+	Cardinality       Cardinality `json:"cardinality"`
+	ParentCardinality Cardinality `json:"parent_cardinality" yaml:"parentCardinality"`
+	Def               string      `json:"def"`
+	Virtual           bool        `json:"virtual"`
 }
 
 type DriverMeta struct {
@@ -167,7 +171,7 @@ func (s *Schema) FindTableByName(name string) (*Table, error) {
 	return nil, errors.Errorf("not found table '%s'", name)
 }
 
-// FindRelation ...
+// FindRelation find relation by columns and parent colums
 func (s *Schema) FindRelation(cs, pcs []*Column) (*Relation, error) {
 L:
 	for _, r := range s.Relations {
@@ -352,6 +356,14 @@ func (s *Schema) Repair() error {
 		if len(t.Constraints) == 0 {
 			t.Constraints = nil
 		}
+		for _, ct := range t.Constraints {
+			if len(ct.Columns) == 0 {
+				ct.Columns = nil
+			}
+			if len(ct.ReferencedColumns) == 0 {
+				ct.ReferencedColumns = nil
+			}
+		}
 		if len(t.Triggers) == 0 {
 			t.Triggers = nil
 		}
@@ -393,6 +405,9 @@ func (s *Schema) Repair() error {
 		}
 		r.ParentTable = pt
 	}
+	if len(s.Functions) == 0 {
+		s.Functions = nil
+	}
 
 	return nil
 }
